<?php defined('SYSPATH') OR die('No direct access allowed.');

/**
 * Controls for export/save status dialog
 */
class Nachos_Controller extends Ninja_Controller {

    /**
    * Renders the view for export save, with banner, visible to everybody,
    * and css
    */
    public function banner () {
        if(Auth::instance()->logged_in() == false) {
            return false;
        }
        $exports = $this->get_exports();
        $this->template->content = new View('banner', array( 'exports' => $exports, 'current_user' => $this->get_current_user() ));
        $this->template->css[] = $this->add_path('../media/css/nachos.css');
    }

    /**
    * Renders banner content.
    */
    public function banner_content($is_config = false) {
        if(Auth::instance()->logged_in() == false) {
            return false;
        }
        $is_config = $this->input->get('is_config', $is_config);
        $exports = $this->get_exports();
        $this->template = new View('banner', array( 'exports' => $exports, 'is_config' => $is_config, 'current_user' => $this->get_current_user() ));
    }

    /**
    * Renders the view for more details, with steps and more information.
    */
    public function details($export_id = 0) {
        $export_id = $this->input->get('export_id', $export_id);
        if($export_id === 0 || empty($export_id)) {
            die;
        }
        $data = $this->get_export_by_id($export_id);
        $this->template = new View('details', array('data' => $data));
    }

    /**
     * Fetches and presents username of current user
     */
    public function username() {
        $username = $this->get_current_user();
        $this->template = new View('data', array('data' => $username));
    }

    /**
    * Fetches and presents export id of latest export for current user
    */
    public function users_latest_export() {
        $export_id = $this->get_latest_export_for_user();
        $this->template = new View('data', array('data' => $export_id));
    }

    /**
    * Connects to API by cURL to get data
    * @return $data array = array()
    */
    public function fetch_data_with_curl($path) {
        $url = 'http://localhost:8009/v1' . $path;

        $curl_object = curl_init();
        curl_setopt($curl_object, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl_object, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl_object, CURLOPT_URL, $url);
        $result = curl_exec($curl_object);
        curl_close($curl_object);

        return json_decode($result, true);
    }

    /**
    * Gets status and id from latest export, to see if we got an export running
    * @return $data array = array()
    */
    public function get_latest_export_details() {
        $data = $this->fetch_data_with_curl('/latestexport');
        return $data;
    }

    /**
     * Gets details about all exports, for banner and more details
     * @return $refined_data array = array()
     */
    public function get_exports() {
        $exports = $this->fetch_data_with_curl('/exports');
        $data = array();

        if(!$exports) {
            return $data;
        }

        $current_user = $this->get_current_user();

        foreach($exports as $export) {
            $export_status = $export['status'];

            if(($export['status'] == 'success' || $export['status'] == 'fail') && $current_user != $export['user']) {
                continue;
            }

            if(isset($data[$export_status]['creation_time'])) {
                if($data[$export_status]['creation_time'] > $export['creation_time']) {
                    continue;
                }
            }

            $data[$export_status] = $export;
            $data[$export_status]['by_me'] = ($current_user == $export['user'] ? true : false);
            $data[$export_status]['all_steps'] = $this->get_all_step_info($export['status_details'], $export['status']);
            $data[$export_status]['current_step_number'] = $this->get_current_step_number($data[$export_status]['all_steps']);
            $data[$export_status]['active_icon_number'] = $this->get_last_active_icon_number($data[$export_status]['all_steps']);
            $number_of_steps = count($data[$export_status]['all_steps']);
            $creation_time = $data[$export_status]['creation_time'];
            $creation_time_format = date('Y-m-d H:i', strtotime($creation_time));

            switch($export['status']) {
                case 'pending':
                case 'running':
                    $current_step = array_slice($data[$export_status]['all_steps'], ($data[$export_status]['active_icon_number'] - 1), 1);
                    $current_step_array = array_values($current_step);
                    $banner_text = ($data[$export_status]['by_me'] ? 'Save in progress' : 'Save in progress. Initiated by ' . $export['user'] );
                    $banner_text .= '. ' . $current_step_array[0]['step_name'];
                    $banner_text .= ' (step ' . $data[$export_status]['active_icon_number'] . ' of ' . $number_of_steps . ')';
                    if(!empty($data[$export_status]['all_steps']['config_generation']['details'])) {
                        $banner_text .= '<br />' . $data[$export_status]['all_steps']['config_generation']['details'];
                    }
                    $data[$export_status]['banner'] = $banner_text;
                    $data[$export_status]['description'] = $banner_text;
                    $data[$export_status]['class'] = 'info-notice info-notice-information';
                    break;
                case 'success':
                    $data[$export_status]['banner'] = ($data[$export_status]['by_me'] ? 'Changes successfully saved (' . $creation_time_format . ')' : 'Changes successfully saved. Save initiated by ' . $export['user'] . '(' . $creation_time_format . ')');
                    $data[$export_status]['description'] = 'Save completed';
                    $data[$export_status]['class'] = 'info-notice info-notice-success';
                    break;
                case 'fail':
                    $data[$export_status]['banner'] = ($data[$export_status]['by_me'] ? 'Changes could not be saved (' . $creation_time_format . ')' : 'Changes could not be saved. Save initiated by ' . $export['user'] . '(' . $creation_time_format . ')');
                    $failed_step = reset(array_slice($data[$export_status]['all_steps'], ($data[$export_status]['active_icon_number'] - 1), 1));
                    $data[$export_status]['description'] = $failed_step['details'];
                    $data[$export_status]['description'] = '';
                    $data[$export_status]['class'] = 'info-notice info-notice-error';
                    break;
                default:
                    $data[$export_status]['banner'] = ucfirst($export['status']);
                    $data[$export_status]['description'] = '';
                    $data[$export_status]['class'] = 'info-notice info-notice-information';
                    break;
            }
        }
        $refined_data = $this->refine_data($data);

        return $refined_data;
    }

    /**
    * Get details about export from id, for banner and more details
    * @return $data array = array()
    */
    public function get_export_by_id($export_id) {
        $data = $this->fetch_data_with_curl('/exports/' . $export_id);
        $data['all_steps'] = $this->get_all_step_info($data['status_details'], $data['status']);
        $data['current_step_number'] = $this->get_current_step_number($data['all_steps']);
        $data['active_icon_number'] = $this->get_last_active_icon_number($data['all_steps']);
        $number_of_steps = count($data['all_steps']);
        $data['by_me'] = op5auth::instance()->get_user()->get_username() == $data['user'];

        switch($data['status']) {
            case 'pending':
            case 'running':
                $banner_text = ($data['by_me'] ? 'Save in progress' : 'Save in progress. Initiated by ' . $data['user'] );
                $banner_text .= ' (step ' . $data['active_icon_number'] . ' of ' . $number_of_steps . ')';
                if(!empty($data['all_steps']['config_generation']['details'])) {
                    $banner_text .= '<br />' . $data['all_steps']['config_generation']['details'];
                }
                $data['banner'] = $banner_text;
                $data['description'] = $banner_text;
                $data['class'] = 'info';
                break;
            case 'success':
                $data['banner'] = ($data['by_me'] ? 'Changes successfully saved' : 'Changes successfully saved. Save initiated by ' . $data['user']);
                $data['description'] = 'Save completed';
                $data['class'] = 'success';
                break;
            case 'fail':
                $data['banner'] = ($data['by_me'] ? 'Changes could not be saved' : 'Changes could not be saved. Save initiated by ' . $data['user']);
                $failed_step = reset(array_slice($data['all_steps'], ($data['active_icon_number'] - 1), 1));
                if(is_array($data['error'])) {
                    $troubleshootingLink = 'https://support.itrsgroup.com/hc/en-us/articles/360020056074-Troubleshooting-Naemon-Configuration-Server' . $data['error']['name'];
                    $data['description'] =  'An error occurred while saving.';
                    $data['description'] .= ' To look for a solution on this error, go to the <a href="' . $troubleshootingLink . '" target="_blank">troubleshooting article.</a>';
                    $data['error_info'] = '<h5>' . $data['error']['name'] . '</h5>';
                    $data['error_info'] .= $data['error']['description'];
                } else {
                    $data['description'] =  'An unknown error occurred while saving. Contact support for further assistance.';
                    $data['error_info'] = $failed_step['details'];
                }
                $data['class'] = 'error';
                break;
            default:
                $data['banner'] = ucfirst($data['status']);
                $data['description'] = '';
                $data['class'] = 'info';
                break;
        }

        return $data;
    }

    /**
    * Get details about latest export for current user
    * @return $export_id string = ""
    */
    public function get_latest_export_for_user() {
        $exports = $this->fetch_data_with_curl('/exports');
        $current_user = $this->get_current_user();
        $export_id = '';

        foreach($exports as $export) {
            if($current_user != $export['user']) {
                continue;
            }

            $export_id = $export['id'];
            break;
        }

        return $export_id;
    }

    /**
    * Adjusts the data to look as we want, and deletes the data we don't need
    * @param $data = array()
    * $return $data = array()
    */
    public function refine_data($data) {
        $current_user = $this->get_current_user();

        if(isset($data['success']) && isset($data['fail'])) {
            if($data['success']['creation_time'] > $data['fail']['creation_time']) {
                unset($data['fail']);
            } else {
                unset($data['success']);
            }
        }

        // To make sure the running banners are presented at the bottom
        $last_in_array = array('pending', 'running');
        foreach($last_in_array as $value) {
            if(isset($data[$value])) {
                $array_post = $data[$value];
                unset($data[$value]);
                $data[$value] = $array_post;

                if($current_user == $data[$value]['user'] && (isset($data['success']) || isset($data['fail']))) {
                    unset($data['success']);
                    unset($data['fail']);
                }
            }
        }
        return $data;
    }

    /**
    * Creates an object with all the steps, even the ones that we haven't gone
    * through with yet.
    * This is where you can add or delete steps.
    * @param $existing_details array = array()
    * @param $status string = ""
    * @return $all_steps array = array()
    */
    public function get_all_step_info($existing_details, $status = "") {
        $step_list = array('backup', 'config_generation', 'verification', 'commit');

        $all_steps = array();
        $step_number = 1;
        foreach($step_list as $step) {
            if(!empty($existing_details)) {
                foreach($existing_details as $details) {
                    if($step == $details['state']) {
                        $all_steps[$step] = $details;
                        $all_steps[$step]['class'] = ($details['progress'] == 1 ? 'success' : '');
                        if($status == 'fail' && ($details['progress'] < 1)) {
                            $all_steps[$step]['icon'] = icon::get('cancel-circled');
                        } else {
                            $all_steps[$step]['icon'] = ($details['progress'] == 1 ? icon::get('state-ok') : $step_number);
                        }
                    }
                }
            } else {
                $all_steps[$step] = array();
                if($status == 'fail') {
                    $all_steps[$step]['icon'] = icon::get('cancel-circled');
                }
            }
            $all_steps[$step]['step_name'] = $this->get_step_name($step);
            $all_steps[$step]['progress'] = (empty($all_steps[$step]['progress']) ? '0' : $all_steps[$step]['progress']);
            $all_steps[$step]['class'] = (empty($all_steps[$step]['class']) ? 'pending' : $all_steps[$step]['class']);
            $all_steps[$step]['icon'] = (empty($all_steps[$step]['icon']) ? $step_number : $all_steps[$step]['icon']);
            $all_steps[$step]['state'] = $step;
            $step_number++;
        }
        return $all_steps;
    }

    /**
     * This is where we set step names from state. Used in lightbox details
     * @param $step string = ""
     * @return $step_name string = ""
     */
    public function get_step_name($step) {
        switch($step) {
            case 'backup':
                $step_name = "Backing up";
                break;
            case 'config_generation':
                $step_name = "Writing configuration";
                break;
            case 'verification':
                $step_name = "Verifying";
                break;
            case "commit":
                $step_name = "Committing";
                break;
            default:
                $step_name = str_replace("_", " ", ucfirst($step));
                break;
        }
        return $step_name;
    }

    /**
    * Fetches the step number where we currently are working.
    * @param $all_steps array = array()
    * @return $current_step int = 0
    */
    public function get_current_step_number($all_steps) {
        $step_number = 0;
        $current_step = 0;
        foreach($all_steps as $step => $details) {
            $step_number++;
            if($details['progress'] < 1) {
                $current_step = $step_number;
            }
        }
        return $current_step;
    }

    /**
    * Fetches the step number where we were working last time.
    * This function doesn't care if we are rollbacking. It fetches the last
    * active step, so that we can see were we were before rollback.
    * @param $all_steps array = array()
    * @return $step_number int = 0
    */
    public function get_last_active_icon_number($all_steps) {
        $icon_number = 0;
        foreach($all_steps as $step => $details) {
            $icon_number++;
            if($details['progress'] < 1) {
                return $icon_number;
            }
        }
        return $icon_number;
    }

    /**
    * Fetches the username of current user
    * @return $username string = ""
    */
    public function get_current_user() {
        $username = Auth::instance()->get_user()->username;
        return $username;
    }
}
